function y = diagm(x)
% Purpose:  defines the diagm operator needed to compute the second-order accurate 
%           approximation of the solution to a DSGE model with time-varying risk. 
%           (see: "Second-Order Approximation of Dynamic Models with Time-Varying Risk", 
%           by Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic, 2010)
% 
% input:    x.  matrix of size (mn x n), such that
% 
%               x = [x1; x2; ... xm] 
% 
%               where each xi is (n x n)
% 
% output:   y.  matrix of size (n x m), such that column i is the main
%               diagonal of submatrix xi
% 
% (c) Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic
% November 2010
% 


n = size(x,2);
m = size(x,1)/n;

y = zeros(n,m);

for i=1:m    
    y(:,i) = diag(x((n*(i-1)+1):i*n,1:n));
end
