function y = diagmn(x)
% Purpose:  defines the diagmn operator needed to compute the second-order accurate 
%           approximation of the solution to a DSGE model with time-varying risk. 
%           (see: "Second-Order Approximation of Dynamic Models with Time-Varying Risk", 
%           by Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic, 2010)
% 
% input:    x.  matrix of size (mn x n), such that
% 
%               x = [x1; x2; ... xm] 
% 
%               where each xi is (n x n)
% 
% output:   y.  matrix of size (mn x n), such that each submatrix i is diag(diag(xi))
% 
% (c) Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic
% November 2010
% 


n = size(x,2);
m = size(x,1)/n;

y = zeros(m*n,n);

for i=1:m    
    y((n*(i-1)+1):i*n,1:n) = diag(diag(x((n*(i-1)+1):i*n,1:n)));
end
