% Purpose:  This code evaluates the analytical first and second 
%           (if approx=2) derivatives of f numerically.
% 
% Note:     The second derivative is the Magnus-Neudecker Hessian matrix of function f
% 
% Calls:    MNhessian.m
% 
% The parameters and steady state values of the arguments of the function f are assumed 
%           to be in the workspace. Also, the order of approximation must be in the workspace.
%
% Adapted from codes written by Stephanie Schmitt-Grohe and Martin Uribe
% 
% (c) Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic
% November 2010
% 

nDf     = zeros(size(Df));
nDf(:)  = eval(Df(:));

    nfypyp    = zeros(size(fypyp));
    nfypy     = zeros(size(fypy));
    nfypxp    = zeros(size(fypxp));
    nfypx     = zeros(size(fypx));

    nfyyp     = zeros(size(fyyp));
    nfyy      = zeros(size(fyy));
    nfyxp     = zeros(size(fyxp));
    nfyx      = zeros(size(fyx));
    
    nfxpyp    = zeros(size(fxpyp));
    nfxpy     = zeros(size(fxpy));
    nfxpxp    = zeros(size(fxpxp));
    nfxpx     = zeros(size(fxpx));
    
    nfxyp     = zeros(size(fxyp));
    nfxy      = zeros(size(fxy));
    nfxxp     = zeros(size(fxxp));
    nfxx      = zeros(size(fxx));
    
if approx==2
   
    nfypyp(:) = eval(fypyp(:));
    nfypy(:)  = eval(fypy(:));
    nfypxp(:) = eval(fypxp(:));
    nfypx(:)  = eval(fypx(:));
    
    nfyyp(:)  = eval(fyyp(:));
    nfyy(:)   = eval(fyy(:));
    nfyxp(:)  = eval(fyxp(:));
    nfyx(:)   = eval(fyx(:));
    
    nfxpyp(:) = eval(fxpyp(:));
    nfxpy(:)  = eval(fxpy(:));
    nfxpxp(:) = eval(fxpxp(:));
    nfxpx(:)  = eval(fxpx(:));
    
    nfxyp(:)  = eval(fxyp(:));
    nfxy(:)   = eval(fxy(:));
    nfxxp(:)  = eval(fxxp(:));
    nfxx(:)   = eval(fxx(:));

    hesypyp = MNhessian(nfypyp);
    hesypy  = MNhessian(nfypy);
    hesypxp = MNhessian(nfypxp);
    hesypx  = MNhessian(nfypx);

    hesyyp  = MNhessian(nfyyp);
    hesyy   = MNhessian(nfyy);
    hesyxp  = MNhessian(nfyxp);
    hesyx   = MNhessian(nfyx);

    hesxpyp = MNhessian(nfxpyp);
    hesxpy  = MNhessian(nfxpy);
    hesxpxp = MNhessian(nfxpxp);
    hesxpx  = MNhessian(nfxpx);

    hesxyp  = MNhessian(nfxyp);
    hesxy   = MNhessian(nfxy);
    hesxxp  = MNhessian(nfxxp);
    hesxx   = MNhessian(nfxx);
    
    nHf =   [hesypyp hesypxp hesypy hesypx; ...
            hesxpyp hesxpxp hesxpy hesxpx; ...
            hesyyp hesyxp hesyy hesyx; ...
            hesxyp hesxxp hesxy hesxx];

end 
